

class HAR:

    def __init__(self, *, log=None, **kwargs):
        self.log = log or Log(**kwargs)

    def __getattr__(self, name):
        return getattr(self.log, name)


class Model:

    def __init__(self, extended_arguments=None, comment=""):
        self.extended_arguments = extended_arguments or {}
        self.comment = comment

    def __eq__(self, other):
        return (self.__class__ == other.__class__ and
                self.__dict__ == other.__dict__)

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, repr(self.__dict__))


class Request(Model):

    def __init__(self, *,
                 method,
                 url,
                 http_version="HTTP/1.0",
                 cookies=None,
                 headers=None,
                 query_string=None,
                 post_data=None,
                 header_size=-1,
                 body_size=-1,
                 **kwargs):

        super().__init__(**kwargs)
        self.method = method
        self.url = url
        self.http_version = http_version
        self.cookies = cookies or []
        self.headers = headers or []
        self.query_string = query_string or []
        self.post_data = post_data or PostData()
        self.header_size = header_size
        self.body_size = body_size


class Response(Model):

    def __init__(self, *,
                 status,
                 status_text,
                 http_version="HTTP/1.0",
                 cookies=None,
                 headers=None,
                 content=None,
                 redirect_url=None,
                 header_size=-1,
                 body_size=-1,
                 **kwargs):

        super().__init__(**kwargs)
        self.status = status
        self.status_text = status_text
        self.http_version = http_version
        self.cookies = cookies or []
        self.headers = headers or []
        self.content = content
        self.redirect_url = redirect_url
        self.header_size = header_size
        self.body_size = body_size


class Cache(Model):

    def __init__(self, *,
                 before_request=None,
                 after_request=None,
                 **kwargs):

        super().__init__(**kwargs)
        self.before_request = before_request
        self.after_request = after_request


class CacheState(Model):

    def __init__(self, *,
                 expires=None,
                 last_access=None,
                 etag=None,
                 hit_count=0,
                 **kwargs):

        self.expires = expires
        self.last_access = last_access
        self.etag = etag
        self.hit_count = hit_count


class Content(Model):

    def __init__(self, *,
                 size=-1,
                 mime_type=None,
                 text="",
                 encoding=None,
                 **kwargs):

        super().__init__(**kwargs)
        self.size = size
        self.mime_type = mime_type
        self.text = text
        self.encoding = encoding


class Timings(Model):

    def __init__(self, *,
                 blocked=-1,
                 dns=-1,
                 connect=-1,
                 send=-1,
                 wait=-1,
                 receive=-1,
                 ssl=-1,
                 **kwargs):

        super().__init__(**kwargs)
        self.blocked = blocked
        self.dns = dns
        self.connect = connect
        self.send = send
        self.wait = wait
        self.receive = receive
        self.ssl = ssl


class Entry(Model):

    def __init__(self, *,
                 pageref=None,
                 started_date_time=None,
                 time=-1,
                 request=None,
                 response=None,
                 cache=None,
                 timings=None,
                 server_ip_address=None,
                 connection=None,
                 **kwargs):

        super().__init__(**kwargs)
        self.pageref = pageref
        self.started_date_time = started_date_time
        self.time = time
        self.request = request
        self.response = response
        self.cache = cache
        self.timings = timings
        self.server_ip_address = server_ip_address
        self.connection = connection


class Header(Model):

    def __init__(self, *, name, value, **kwargs):
        super().__init__(**kwargs)
        self.name = name
        self.value = value


class Cookie(Model):
    def __init__(self, *,
                 name,
                 value,
                 path=None,
                 domain=None,
                 expires=None,
                 http_only=False,
                 secure=False,
                 **kwargs):

        super().__init__(**kwargs)
        self.name = name
        self.value = value
        self.path = path
        self.domain = domain
        self.expires = expires
        self.http_only = http_only
        self.secure = secure


class PostData(Model):

    def __init__(self, *,
                 mime_type=None,
                 params=None,
                 text="",
                 **kwargs):
        super().__init__(**kwargs)
        self.mime_type = mime_type
        self.params = params or []
        self.text = text


class Param(Model):

    def __init__(self, *,
                 name,
                 value,
                 **kwargs):
        super().__init__(**kwargs)
        self.name = name
        self.value = value


class PostParam(Model):

    def __init__(self, *,
                 name,
                 value,
                 file_name=None,
                 content_type=None,
                 **kwargs):
        super().__init__(**kwargs)
        self.name = name
        self.value = value
        self.file_name = file_name
        self.content_type = content_type


class Creator(Model):

    def __init__(self, *,
                 name,
                 version,
                 **kwargs):
        super().__init__(**kwargs)
        self.name = name
        self.version = version


class Page(Model):

    def __init__(self, *,
                 id,
                 title,
                 started_date_time=None,
                 page_timings=None,
                 **kwargs):
        super().__init__(**kwargs)
        self.id = id
        self.title = title
        self.started_date_time = started_date_time
        self.page_timings = page_timings


class PageTimings(Model):

    def __init__(self, *,
                 on_content_load=-1,
                 on_load=-1,
                 **kwargs):
        super().__init__(**kwargs)
        self.on_content_load = on_content_load
        self.on_load = on_load


class Browser(Model):

    def __init__(self, *,
                 name,
                 version,
                 **kwargs):
        super().__init__(**kwargs)
        self.name = name
        self.version = version


class Log(Model):

    def __init__(self, *,
                 version="1.1",
                 creator=None,
                 browser=None,
                 pages=None,
                 entries=None,
                 **kwargs):
        super().__init__(**kwargs)
        self.version = version
        self.creator = creator
        self.browser = browser
        self.pages = pages or []
        self.entries = entries or []



from marshmallow import Schema as BaseSchema, fields, post_dump, post_load
from .model import Cookie, Request, Response, Entry, HAR, Header, Param, PostData, PostParam
from .model import Log, Page, Creator, Browser, Cache, CacheState, Timings, Content, PageTimings


class Schema(BaseSchema):

    extended_arguments = fields.Raw(required=False)
    comment = fields.String(required=False)

    @post_load(pass_original=True)
    def load_extended(self, data, original_data):
        if isinstance(original_data, dict):
            extended_arguments = {k: v for k, v in original_data.items() if k.startswith('_')}

            if isinstance(data, dict):
                data["extended_arguments"] = extended_arguments

        return self.__model__(**data)

    @post_dump
    def dump_extended(self, data):
        extended = data.pop("extended_arguments", {})
        data.update(extended)


class CookieSchema(Schema):

    __model__ = Cookie

    DATE_FORMAT = "iso"

    name = fields.String()
    value = fields.String()
    path = fields.String(required=False, allow_none=True)
    domain = fields.String(required=False, allow_none=True)
    expires = fields.DateTime("iso", required=False, allow_none=True)
    http_only = fields.Boolean(required=False, load_from="httpOnly", dump_to="httpOnly")
    secure = fields.Boolean(required=False)


class HeaderSchema(Schema):

    __model__ = Header

    name = fields.String(required=True)
    value = fields.String(required=True)


class ParamSchema(Schema):

    __model__ = Param

    name = fields.String(required=True)
    value = fields.String(required=True)


class PostParamSchema(Schema):

    __model__ = PostParam

    name = fields.String(required=True)
    value = fields.String(required=True)
    file_name = fields.String(required=False, load_from="fileName", dump_to="fileName")
    content_type = fields.String(required=False, load_from="contentType", dump_to="contentType")


class PostDataSchema(Schema):

    __model__ = PostData

    mime_type = fields.String(required=False, load_from="mimeType", dump_to="mimeType")
    params = fields.Nested(PostParamSchema, many=True)
    text = fields.String(required=False)


class RequestSchema(Schema):

    __model__ = Request

    method = fields.String(required=True)
    url = fields.Url(required=True)
    http_version = fields.String(required=False, load_from="httpVersion", dump_to="httpVersion")
    cookies = fields.Nested(CookieSchema, many=True)
    headers = fields.Nested(HeaderSchema, many=True)
    query_string = fields.Nested(ParamSchema, many=True, load_from="queryString", dump_to="queryString")
    post_data = fields.Nested(PostDataSchema, required=False, load_from="postData", dump_to="postData")
    header_size = fields.Integer(required=False, load_from="headerSize", dump_to="headerSize")
    body_size = fields.Integer(required=False, load_from="bodySize", dump_to="bodySize")


class ContentSchema(Schema):

    __model__ = Content

    size = fields.Integer(required=False, default=-1)
    mime_type = fields.String(required=False, allow_none=True, load_from="mimeType", dump_to="mimeType")
    text = fields.String(required=True)
    encoding = fields. String(required=False, allow_none=True)


class ResponseSchema(Schema):

    __model__ = Response

    status = fields.Integer(required=True)
    status_text = fields.String(required=True, load_from="statusText", dump_to="statusText")
    http_version = fields.String(required=False, load_from="httpVersion", dump_to="httpVersion")
    cookies = fields.Nested(CookieSchema, many=True)
    headers = fields.Nested(HeaderSchema, many=True)
    content = fields.Nested(ContentSchema, required=False, allow_none=True)
    redirect_url = fields.String(required=False, allow_none=True, load_from="redirectURL", dump_to="redirectURL")
    header_size = fields.Integer(required=False, default=-1, load_from="headerSize", dump_to="headerSize")
    body_size = fields.Integer(required=False, default=-1, load_from="bodySize", dump_to="bodySize")


class CacheStateSchema(Schema):

    __model__ = CacheState

    expires = fields.DateTime("iso", required=False, allow_none=True)
    last_access = fields.DateTime("iso", required=False, allow_none=True, load_from="lastAccess", dump_to="lastAccess")
    etag = fields.String(required=False, allow_none=True, load_from="eTag", dump_to="eTag")
    hit_count = fields.Integer(required=False, load_from="hitCount", dump_to="hitCount")


class CacheSchema(Schema):

    __model__ = Cache

    before_request = fields.Nested(CacheStateSchema, required=False, allow_none=True,
                                   load_from="beforeRequest", dump_to="beforeRequest")
    after_request = fields.Nested(CacheStateSchema, required=False, allow_none=True,
                                  load_from="afterRequest", dump_to="afterRequest")


class TimingsSchema(Schema):

    __model__ = Timings

    blocked = fields.Integer(required=False, default=-1)
    dns = fields.Integer(required=False, default=-1)
    connect = fields.Integer(required=False, default=-1)
    send = fields.Integer(required=False, default=-1)
    wait = fields.Integer(required=False, default=-1)
    receive = fields.Integer(required=False, default=-1)
    ssl = fields.Integer(required=False, default=-1)


class EntrySchema(Schema):

    __model__ = Entry

    pageref = fields.String(required=False, allow_none=True)
    started_date_time = fields.DateTime("iso", required=False, allow_none=True,
                                        load_from="startedDateTime", dump_to="startedDateTime")
    time = fields.Integer(required=False, default=-1)
    request = fields.Nested(RequestSchema, required=False, allow_none=True)
    response = fields.Nested(ResponseSchema, required=False, allow_none=True)
    cache = fields.Nested(CacheSchema, required=False, allow_none=True)
    timings = fields.Nested(TimingsSchema, required=False, allow_none=True)
    server_ip_address = fields.String(required=False, allow_none=True,
                                      load_from="serverIPAddress", dump_to="serverIPAddress")
    connection = fields.String(required=False, allow_none=True)


class CreatorSchema(Schema):

    __model__ = Creator

    name = fields.String(required=True)
    version = fields.String(required=True)


class BrowserSchema(Schema):

    __model__ = Browser

    name = fields.String(required=True)
    version = fields.String(required=True)


class PageTimingsSchema(Schema):

    __model__ = PageTimings

    on_content_load = fields.Integer(required=False, default=-1, load_from="onContentLoad", dump_to="onContentLoad")
    on_load = fields.Integer(required=False, default=-1, load_from="onLoad", dump_to="onLoad")


class PageSchema(Schema):

    __model__ = Page

    id = fields.String(required=True)
    title = fields.String(required=True)
    started_date_time = fields.DateTime("iso", required=False, allow_none=True,
                                        load_from="startedDateTime", dump_to="startedDateTime")
    page_timings = fields.Nested(PageTimingsSchema, required=False, allow_none=True,
                                 load_from="pageTimings", dump_to="pageTimings")


class LogSchema(Schema):

    __model__ = Log

    version = fields.String(required=False)
    creator = fields.Nested(CreatorSchema, required=False, allow_none=True)
    browser = fields.Nested(BrowserSchema, required=False, allow_none=True)
    pages = fields.Nested(PageSchema, many=True)
    entries = fields.Nested(EntrySchema, many=True)


class HARSchema(BaseSchema):

    __model__ = HAR

    log = fields.Nested(LogSchema, required=True)

    @post_load
    def load_extended(self, data):
        return self.__model__(**data)



#!/usr/bin/env python

from setuptools import setup

setup(name='marshmallow-har',
      version='0.2',
      description='Simple set of marshmallow schemas to load/dump the HTTP Archive (HAR) format.',
      author='Delve Labs inc.',
      author_email='info@delvelabs.ca',
      url='https://github.com/delvelabs/marshmallow-har',
      packages=['marshmallow_har'],
      install_requires=[
          'marshmallow',
      ])
